<?php
/**
 * @file
 * Specific functions to make user answers human readable.
 */

/**
 * Returns a mapping between (e.g. multiple choice) CQ answer ids and the actual answer.
 *
 * @staticvar array $closedquestion_scoreboard_cq_answer_mapping
 *
 * @param array $cq_nids
 *   An array of Closed Question nids.
 *
 * @param boolean $reset
 *   Set to TRUE to skip/rebuild the Drupal cache.
 *
 * @return array
 *   A hash:
 *   array(<nid> => array(
 *     'cq_type'=> Closed Question type,
 *     'mapping'=>array(
 *       <answer_id> => Answer text,
 *       ...
 *     ),
 *     ...
 *   )
 */
function closedquestion_scoreboard_get_cq_rendered_answer_mapping($cq_nids, $reset = TRUE) {
  static $closedquestion_scoreboard_cq_answer_mapping = array();

  /**
   * Helper function to get xml node attribute value
   */
  $_getAttValue = function ($xmlNode, $attName) {
    $val = '';
    $atts = (array) $xmlNode->attributes();
    if (isset($atts['@attributes'][$attName])) {
      $val = $atts['@attributes'][$attName];
    }
    return $val;
  };

  /**
   * Helper function which turns CQ xml node into answer mapping.
   */
  $_getMapping = function ($xmlNode, $attName = 'id') use ($_getAttValue) {
    $mapping = array();

    $attValue = $_getAttValue($xmlNode, $attName);
    if ($attValue !== '') {
      $key = $attValue;
      $value = (strip_tags($xmlNode->asXml()));
      $mapping[$key] = $value;
    }

    return $mapping;
  };

  /**
   * For each question we want to obtain the following array:
   *
   * array(<nid> => array(
   *   'cq_type'=> Closed Question type,
   *   'mapping'=>array(
   *     <answer_id> => Answer text,
   *     ...
   *   )
   * );
   * */
  $numberOfCqs = count($cq_nids);
  for ($i = 0; $i < $numberOfCqs; $i++) {
    $nid = $cq_nids[$i];

    if (isset($closedquestion_scoreboard_cq_answer_mapping[$nid])) {
      continue; // Mapping already exists.
    }

    /* Add a new mapping */

    // Add it from Drupal cache.
    $cache = $reset === FALSE ? cache_get('closedquestion_scoreboard_cq_mapping_node_' . $nid) : FALSE;
    if ($reset === FALSE && !empty($cache->data)) {
      $mapping = unserialize($cache->data);
    }
    else {
      // No cache. Add it from the Closed Question.
      $cq_node = node_load($nid);
      if ($cq_node->type !== 'closedquestion') {
        drupal_set_message('Warning: Skipped node with id ' . $nid . ' because it is not a Closed Question', 'warning');
        continue;
      }
      $cq_body = field_get_items('node', $cq_node, 'body');

      if (isset($cq_body[0]['value'])) {
        $cq_xml_str = $cq_body[0]['value'];
        $cq_xml_obj = simplexml_load_string($cq_xml_str);

        $atts = (array) $cq_xml_obj->attributes();
        $cq_type = strtolower($atts['@attributes']['type']);
        $mapping[$nid]['cq_type'] = $cq_type;
        $mapping[$nid]['mapping'] = array();

        // Obtain mapping based on the typical XML structure of the question type.
        switch ($cq_type) {
          case 'arrow':
            foreach ($cq_xml_obj->matchimg->children() as $second_gen) {
              if ($second_gen->getName() === 'hotspot') {
                $mapping[$nid]['mapping'] = $mapping[$nid]['mapping'] + $_getMapping($second_gen);
              }
            }
            break;

          case 'check':
          case 'option':
            $j = 0;
            foreach ($cq_xml_obj->children() as $second_gen) {
              if ($second_gen->getName() === 'option') {
                if ($cq_type === 'check') {
                  $answer_id = $_getAttValue($second_gen, 'id');
                  if ($answer_id === '') {
                    $answer_id = 'o' . $j;
                  }
                }
                else {
                  $answer_id = $j; //'option' Closed Question type
                }

                $value = strip_tags($second_gen->choice->asXml());
                $mapping[$nid]['mapping'] = $mapping[$nid]['mapping'] + array($answer_id => $value);
                $j++;
              }
            }
            break;

          case 'fillblanks':
            foreach ($cq_xml_obj->children() as $second_gen) {
              if ($second_gen->getName() === 'inlineoption') {
                $mapping[$nid]['mapping'] = $mapping[$nid]['mapping'] + $_getMapping($second_gen);
              }
            }
            break;

          case 'selectorder':
            foreach ($cq_xml_obj->children() as $second_gen) {
              if ($second_gen->getName() === 'option') {
                $answer_id = $_getAttValue($second_gen, 'id');
                $value = strip_tags($second_gen->choice->asXml());
                $mapping[$nid]['mapping'] = $mapping[$nid]['mapping'] + array($answer_id => $value);
              }
            }
            break;
        }

        /* Store new mapping in Drupal cache. */
        if (!empty($mapping[$nid])) {
          cache_set('closedquestion_scoreboard_cq_mapping_node_' . $nid, serialize($mapping[$nid]));
        }
      }
    }

    $closedquestion_scoreboard_cq_answer_mapping = $closedquestion_scoreboard_cq_answer_mapping + $mapping;
  }

  return $closedquestion_scoreboard_cq_answer_mapping;
}

/**
 * Processes an answer.
 *
 * @param string $answer
 *
 * @param array $cq_answer_mapping
 *   See @return of function closedquestion_scoreboard_get_cq_rendered_answer_mapping. This
 *   function only needs the mapping of the answer's Closed Question.
 *
 * @return string
 */
function closedquestion_scoreboard_humanreadable_answer($answer, $cq_answer_mapping) {
  $processedAnswer = $answer;
  $processedAnswerAsArray = array();

  // Note: this should be implemented by closedquestion_scoreboard_get_cq_rendered_answer_mapping
  switch ($cq_answer_mapping['cq_type']) {
    case 'arrow':
      $i = 0;
      $pairs = explode(',', $answer);
      foreach ($pairs as $pair) {
        $processedAnswerAsArray[$i] = $cq_answer_mapping['mapping'][$pair[0]];
        $processedAnswerAsArray[$i] .= '&rarr;';
        $processedAnswerAsArray[$i] .= $cq_answer_mapping['mapping'][$pair[1]] . ' ';
        $i++;
      }
      $processedAnswer = implode(t(', '), $processedAnswerAsArray);
      break;

    case 'selectorder':
      $answerLength = strlen($answer);
      for ($i = 0; $i < $answerLength; $i++) {
        if (is_numeric($answer[$i])) {
          $processedAnswerAsArray[] = '<em>' . $cq_answer_mapping['mapping'][$answer[$i]] . '</em>: ';
        }
        else {
          $processedAnswerAsArray[] = $cq_answer_mapping['mapping'][$answer[$i]] . ($i !== $answerLength - 1 ? ', ' : '');
        }
      }
      $processedAnswer = implode(t(' '), $processedAnswerAsArray);
      break;

    case 'fillblanks':
    case 'check':
    case 'option':
      if (isset($cq_answer_mapping['mapping'][$answer])) {
        $processedAnswer = $cq_answer_mapping['mapping'][$answer];
      }
      break;
  }

  return $processedAnswer;
}

/**
 * Implements hook_node_save. This will clear the cached answer mappings.
 */
function closedquestion_scoreboard_node_update($node) {
  if ($node->type === 'closedquestion' && isset($node->nid)) {
    // Invalidate cache.
    cache_clear_all('closedquestion_scoreboard_cq_mapping_node_' . $node->nid, 'cache');
  }
}

/**
 * Implements hook_node_delete. This will clear the cached answer mappings.
 * @param type $node
 */
function closedquestion_scoreboard_node_delete($node) {
  closedquestion_scoreboard_node_update($node); // call hook_node_save implementation.
}
